/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://eu-egee.org/partners/ for details on the copyright holders.
 * For license conditions see the license file or http://eu-egee.org/license.html
 */

/* NB. This header retains some old definitions for backwards compatiblity.
   To use these old definitions, you need to compile with -DRGMA_V4_COMPAT and
   link with the 4.2.0 version of the library. Support for these old
   definitions will be removed in the next release. */

#ifndef RGMA_H
#define RGMA_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>  /* for FILE */

/* GLOBAL CONSTANTS ***********************************************************/

#define RGMAQueryType_LATEST                  1  /* Latest query type. */
#define RGMAQueryType_HISTORY                 2  /* History query type. */
#define RGMAQueryType_CONTINUOUS              4  /* Continuous query type. */
#define RGMAQueryType_STATIC                  8  /* Static query type. */
#define RGMAQueryType_INTERVAL                16 /* Time-interval modifier. */

#define RGMAStorageType_MEMORY                0 /* Memory storage type. */
#define RGMAStorageType_FILE                  1 /* deprecated (don't use it) */
#define RGMAStorageType_DATABASE              2 /* Database storage type. */

#define RGMAProducerProps_LATEST              1 /* Supports latest queries. */
#define RGMAProducerProps_HISTORY             2 /* Supports history queries. */
#define RGMAProducerProps_IGNORESLOWCONSUMERS 4 /* deprecated (don't use it) */
#define RGMAProducerProps_CONTINUOUS          8 /* Supports continuous qs. */
#define RGMAProducerProps_STATIC              16 /* Supports static queries. */
#define RGMAProducerProps_SECONDARYPRODUCER   32 /* Is a Secondary Producer. */

#define RGMAColumnType_INTEGER                0 /* INTEGER data type. */
#define RGMAColumnType_REAL                   1 /* REAL data type. */
#define RGMAColumnType_DOUBLE                 2 /* DOUBLE data type. */
#define RGMAColumnType_TIMESTAMP              3 /* TIMESTAMP data type. */
#define RGMAColumnType_CHAR                   4 /* CHAR data type. */
#define RGMAColumnType_VARCHAR                5 /* VARCHAR data type. */
#define RGMAColumnType_DATE                   6 /* DATE data type. */
#define RGMAColumnType_TIME                   7 /* TIME data type. */

#define RGMAResourceType_UNSET                0 /* Unset (not connected). */
#define RGMAResourceType_CONSUMER             1 /* Consumer resource. */
#define RGMAResourceType_PRIMARYPRODUCER      2 /* Primary Producer resource. */
#define RGMAResourceType_SECONDARYPRODUCER    3 /* Secondary Producer res. */
#define RGMAResourceType_ONDEMANDPRODUCER     4 /* On-demand Producer res. */

#define RGMAService_PRIMARYPRODUCER           0 /* Primary Producer Service */
#define RGMAService_SECONDARYPRODUCER         1 /* Secondary Producer Service */
#define RGMAService_ONDEMANDPRODUCER          2 /* On-demand Producer Service */
#define RGMAService_CONSUMER                  3 /* Consumer Service */
#define RGMAService_REGISTRY                  4 /* Registry Service */
#define RGMAService_SCHEMA                    5 /* Schema Service */
#define RGMAService_MEDIATOR                  6 /* Mediator Service */

#define RGMA_REMOTEEXCEPTION                  1 /* Service connection error. */
#define RGMA_UNKNOWNRESOURCEEXCEPTION         2 /* No such resource error. */
#define RGMA_APIEXCEPTION                     3 /* deprecated */
#define RGMA_EXCEPTION                        4 /* Other R-GMA error. */

#define RGMA_UNCLASSIFIEDEXCEPTION            0 /* Sub-exception. */
#define RGMA_SECURITYEXCEPTION                1 /* Sub-exception. */
#define RGMA_USEREXCEPTION                    2 /* Sub-exception. */
#define RGMA_BUFFERFULLEXCEPTION              3 /* Sub-exception. */
#define RGMA_NOWORKINGREPLICASEXCEPTION		  7 /* Sub-exception. */
#define RGMA_BUSYEXCEPTION					  9 /* Sub-exception. */

/* DEPRECATED CONSTANTS *******************************************************/

#ifdef RGMA_V4_COMPAT
#define RGMAStorageType_VALID(t) ((t) >= 0 && (t) <= 2)  /* deprecated */
#define RGMAColumnType_VALID(t) ((t) >= 0 && (t) <= 7)   /* deprecated */
#define RGMAResourceType_VALID(t) ((t) >= 1 && (t) <= 4) /* deprecated */
#endif /* RGMA_V4_COMPAT */


/* GLOBAL TYPE DEFINITIONS ****************************************************/

/** The RGMAResource structure represents a single producer or consumer
 *  resource, and holds all API settings related to the resource. This
 *  includes the resource-endpoint, the handle to the gSOAP
 *  environment, the resource type and any errors associated with the
 *  resource. It is an opaque structure and you are not expected to read or
 *  change its contents. You should free it when no longer required by calling
 *  RGMA_freeResource.
 */

typedef struct RGMAResource_S RGMAResource;


/** The RGMAResourceEndpoint structure represents a resource-endpoint for an
 *  R-GMA resource (consumer or producer instance). You can obtain a copy
 *  of the resource-endpoint for a resource by calling RGMA_getEndpoint. You
 *  can use the RGMAResourceEndpoint to reconnect to a remote resource (see
 *  RGMA_reconnect). You should free the memory associated with the 
 *  resource-endpoint using RGMA_freeEndpoint, when you have finished with it.
 */

typedef struct
{
	/** Web Service Resource Identifier. */      /* Doxygen requires    */
	int resourceId;                              /* the comments to     */
	/** Web Service URL. */                      /* precede the fields. */
	char *url;                                   
} RGMAResourceEndpoint;


#ifndef RGMA_V4_COMPAT /* v5 version */
/** The RGMAException structure holds an R-GMA exception (of any type).
 *  Exceptions are created by R-GMA services when an operation fails, and
 *  by the API if a local error occurs. You can obtain a pointer to the most
 *  recent exception in a resource, by calling RGMA_getException.
 *
 *  The errorType field will be one of the following:
 *
 *  o RGMA_REMOTEEXCEPTION           (errors connecting to service)       \n
 *  o RGMA_UNKNOWNRESOURCEEXCEPTION  (requested resource does not exist)  \n
 *  o RGMA_EXCEPTION                 (all other RGMA errors)              \n
 *  (the RGMA_APIEXCEPTION type is no longer used)                        \n
 *
 *  The errorNumber field can provide more information that users might wish
 *  to use in their code when errorType is RGMA_EXCEPTION (only). It can take
 *  the following values:
 *
 *  o RGMA_SECURITYEXCEPTION         (authentication or authorization failed) \n
 *  o RGMA_USEREXCEPTION             (invalid parameters, invalid tuple data or
 *                                    resource in wrong state for operation) \n
 *  o RGMA_BUFFERFULLEXCEPTION       (producer's buffer is full) \n
 *  o RGMA_UNCLASSIFIEDEXCEPTION     (some other error - see errorMessage) \n
 *
 *  The errorMessage field will describe the error, and will never be NULL.
 *
 *  The numSuccessfulOps field shows the number of operations that completed
 *  successfully before the exception was raised. It will only be non-zero in
 *  batch operations that have partially succeeded - currently, this means just
 *  RGMAPrimaryProducer_insertList().
 *
 *  Exception structures are declared in the user's code: the user passes
 *  the address of an RGMAException strucure to the API. The API guarantees to
 *  fill in the exception details if the API call returns a failure status.
 *  You can free the memory used to fill in an exception by calling
 *  RGMA_freeException (you should NOT call this if the API call succeeds).
 */

typedef struct
{
	/** Exception type. */
	int errorType;
	/** Error code. */
	int errorNumber;
	/** Error message. */
	char *errorMessage;
	/** Number of operations that completed successfully. */
	int numSuccessfulOps;
} RGMAException;
#endif /* RGMA_V4_COMPAT */


#ifndef RGMA_V4_COMPAT /* v5 version */
/** The RGMAStorageLocation structure holds file or database connection
 *  details for a Primary or Secondary Producer. It is filled in by hand
 *  then passed into the constructor for the producer. Naming your own database
 *  (via the location, username and password fields) is deprecated - you should
 *  assign a logical name instead (via the logicalName field), then call
 *  RGMAPrimaryProducer_listTupleStores() to retrieve the name of the database
 *  created by R-GMA.
 */
                                                                                
typedef struct
{
        /** @deprecated Use logicalName. */
        char *location;
        /** @deprecated Use logicalName. */
        char *username;
        /** @deprecated Use logicalName. */
        char *password;
	/** Logical name for tuple store. */
	char *logicalName;
} RGMAStorageLocation;
#endif /* RGMA_V4_COMPAT */


/** The RGMAStringList structure holds a (possibly empty) list of strings.
 *
 *  It is allocated on the heap and you should call RGMA_freeStringList to
 *  free it when it is no longer required.
 */

typedef struct
{
        /** Number of strings in the list. */
	int numStrings;
        /** List of strings. */
	char **string;
} RGMAStringList;


/** The RGMARow structure holds a single row in an RGMAResultSet. The
 *  interpretation of the value is given in the columnTypes array in the
 *  ResultSet. A NULL pointer is used to represent a database NULL for any
 *  value.
 */

typedef struct
{
	/** Array of column values (INDIVIDUAL VALUES MAY BE NULL). */
	char **cols;
} RGMARow;


#ifndef RGMA_V4_COMPAT /* v5 version */
/** The RGMAResultSet structure holds the set of tuples (database rows)
 *  returned by a consumer query (RGMAConsumer_pop).
 *
 *  In an empty result set, the number of rows and columns will be zero, and
 *  the rows, tableNames, columnNames and columnTypes arrays will all be NULL.
 *  Otherwise, the arrays will all be non-NULL and contain the correct number
 *  of values (numRows for the rows array and numCols for the others).
 *
 *  The tableNames and columnNames arrays will never contain NULL values, but
 *  may contain empty strings. The columnTypes array will always contain valid
 *  values. Possible column types are: RGMAColumnType_INTEGER,
 *  RGMAColumnType_REAL, RGMAColumnType_DOUBLE, RGMAColumnType_TIMESTAMP,
 *  RGMAColumnType_CHAR, RGMAColumnType_VARCHAR, RGMAColumnType_DATE and
 *  RGMAColumnType_TIME.
 *
 *  Warnings, if any, are attached to a result set by the Registry Mediator.
 *  The warning field will never be NULL, but may be an empty string.
 * 
 *  The endOfResults flag in the result set is set to 1 (true) when there are
 *  no more tuples to be returned for a query. Otherwise it will be 0 (false).
 *
 *  The result set is allocated on the heap, and you should free it by calling
 *  RGMA_freeResultSet when it is no longer required. You can also detach parts
 *  of a result set for use in your own code, by changing the corresponding
 *  pointer in the result set to NULL (RGMA_freeResultSet safely skips over
 *  NULL pointers in any part of a result set).
 */

typedef struct
{
	/** Number of rows. */
	int numRows;
	/** Number of columns. */
	int numCols;
	/** Array of column name prefixes. */
	char **tableNames;
	/** Array of column names. */
	char **columnNames;
	/** Array of column types. */
	int *columnTypes;
	/** Array of rows. */
	RGMARow *rows;
	/** Mediator Warning message. */
	char *warningMessage;
	/** End-of-results flag. */
	int endOfResults;
} RGMAResultSet;
#endif /* RGMA_V4_COMPAT */


/** The RGMATupleStore structure holds details of a user's permanent tuple
 *  store. The type field will be RGMAStorageType_DATABASE. Tuple stores are
 *  always created by R-GMA. Users who wish to access a database tuple store
 *  from outside of R-GMA should call RGMAPrimaryProducer_listTupleStores or
 *  RGMASecondaryProducer_listTupleStores to obtain the database URL, then
 *  contact their system administrator to get direct access to the database.
 */

typedef struct
{
	/** Storage type. */
	int type;
	/** Logical name. */
	char *logicalName;
	/** Storage location (JDBC database URL) . */
	char *location;
} RGMATupleStore;


/** The RGMATupleStoreList structure holds a list of RGMATupleStore structures.
 *  It may be empty.
 *
 *  It is allocated on the heap and you should call RGMA_freeTupleStoreList to
 *  free it when it is no longer required.
 */

typedef struct
{
	/** Length of list. */
	int numTupleStores;
	/** List of tuple stores. */
        RGMATupleStore *tupleStore;
} RGMATupleStoreList;


/** The RGMAProducerTableEntry structure holds details of a producer resource's
 *  registry entry for a specified table. */

typedef struct
{
	/** Producer resource's endpoint. */
	RGMAResourceEndpoint *endpoint;
	/** Table name. */
	char *tableName;
	/** Producer type - combination (bitwise OR) of:   \n
	        RGMAProducerProps_LATEST,                  \n
	        RGMAProducerProps_HISTORY,                 \n
	        RGMAProducerProps_CONTINUOUS,              \n
	        RGMAProducerProps_STATIC or                \n
	        RGMAProducerProps_SECONDARYPRODUCER.       \n */
	int producerType;
	/** History Retention Period (in seconds) for this table. */
	int hrpSec;
	/** Quality of service attibute (may be NULL). */
	char *qosAttrib;
} RGMAProducerTableEntry;


/** The RGMAProducerTableEntryList structure holds a list of
 *  RGMAProducerTableEntry structures. It may be empty.
 *
 *  It is allocated on the heap and you should call
 *  RGMA_freeProducerTableEntryList to free it when it is no longer required.
 */

typedef struct
{
	/** Length of list. */
	int numProducerTableEntries;
	/** List of producer/table entries. */
        RGMAProducerTableEntry *producerTableEntry;
} RGMAProducerTableEntryList;


/** The RGMAIndex structure holds the definition of a single table index.
 */

typedef struct
{
	/** Index name. */
        char *indexName;
	/** Number of columns in index. */
	int numColumns;
	/** List of columns in index. */
        char **column;
} RGMAIndex;


/** The RGMAIndexList structure holds a list of RGMAIndex structures.
 *  It may be empty.
 *
 *  It is allocated on the heap and you should call RGMA_freeIndexList to
 *  free it when it is no longer required.
 */

typedef struct
{
	/** Length of list. */
	int numIndexes;
	/** List of tuple stores. */
        RGMAIndex *index;
} RGMAIndexList;


/** The RGMAColumnDefinition structure holds the definition of a single column.
 *
 *  Possible column types are: RGMAColumnType_INTEGER,
 *  RGMAColumnType_REAL, RGMAColumnType_DOUBLE, RGMAColumnType_TIMESTAMP,
 *  RGMAColumnType_CHAR, RGMAColumnType_VARCHAR, RGMAColumnType_DATE and
 *  RGMAColumnType_TIME.
 */

typedef struct
{
        /** Column name. */
	char *name;
        /** Column type. */
	int type;
        /** Column width (used with TIME, TIMESTAMP, CHAR and VARCHAR only). */
	int size;
        /** NOT NULL flag (1=true, 0=false). */
	int notNull;
        /** NOT NULL flag (1=true, 0=false). */
	int primaryKey;
} RGMAColumnDefinition;


/** The RGMATableDefinition structure holds the column definitions for a table.
 *
 *  It is allocated on the heap and you should call RGMA_freeTableDefinition to
 *  free it when it is no longer required.
 */

typedef struct
{
        /** Table name. */
	char *tableName;
        /** Number of columns in the table. */
	int numColumns;
        /** List of column definitions. */
	RGMAColumnDefinition *column;
} RGMATableDefinition;


/* DEPRECATED TYPE DEFINITIONS ************************************************/

#ifdef RGMA_V4_COMPAT

/** @deprecated Use the new version of this structure. */
typedef struct RGMAException_S
{
	int errorType;
	int errorNumber;
	char *errorMessage;
	struct RGMAException_S *next;
} RGMAException;

/** @deprecated Use the new version of this structure. */
typedef struct RGMAStorageLocation_S
{
        char *location;
        char *username;
        char *password;
} RGMAStorageLocation;

/** @deprecated Use the new version of this structure. */
typedef struct RGMAResultSet_S
{
	int numRows;
	int numCols;
	char **tableNames;
	char **columnNames;
	int *columnTypes;
	RGMARow *rows;
	char *warningMessage;
} RGMAResultSet;

/** @deprecated This structure was never used. */

typedef struct RGMATableAuthorization_S
{
	int num_rules;
	char **rules;
} RGMATableAuthorization;

#endif /* RGMA_V4_COMPAT */


/* PUBLIC FUNCTIONS ***********************************************************/

/**
 *  Creates a new Consumer resource.
 *
 *  Extracts the URL of the Consumer Service from the R-GMA
 *  configuration file $RGMA_HOME/etc/rgma/rgma.conf. Both the environment
 *  variable, and the file, must exist.
 *
 *  Continuous, latest and history queries may optionally take a time interval
 *  to limit the age of tuples returned. This is indicated by combining the
 *  query type with the RGMAQueryType_INTERVAL qualifier, for example:
 *  (RGMAQueryType_CONTINUOUS | RGMAQueryType_INTERVAL).
 *
 *  @param  terminationInterval The interval (in seconds) after which this
 *                              Consumer resource will be destroyed by the
 *                              Consumer Service, if there has been no contact
 *                              from the API.
 *  @param  selectStatement     The SQL SELECT statement for this Consumer.
 *  @param  queryType           The type of query. Exactly one of:     \n
 *                                 RGMAQueryType_CONTINUOUS,           \n
 *                                 RGMAQueryType_LATEST,               \n
 *                                 RGMAQueryType_HISTORY or            \n
 *                                 RGMAQueryType_STATIC.               \n
 *                              The first three types may also be combined with
 *                              RGMAQueryType_INTERVAL (see above).
 *  @param  timeInterval	Optional time-interval (in seconds) limiting
 *                              the age of tuples returned by the query.
 *  @param  resourcePP		On output, will contain pointer to new Consumer.
 *
 *  @return 0 on success, -1 on error.
 */

extern int RGMA_createConsumer(int terminationInterval, const char *selectStatement, int queryType, int timeInterval, RGMAResource **resourcePP);


#ifndef RGMA_V4_COMPAT /* v5 version */
/**
 *  Creates a new Primary Producer resource.
 *
 *  Extracts the URL of the Primary Producer Service from the R-GMA
 *  configuration file $RGMA_HOME/etc/rgma/rgma.conf. Both the environment
 *  variable, and the file, must exist.
 *
 *  @param  terminationInterval The interval (in seconds) after which this
 *                              Primary Producer resource will be destroyed by
 *                              Primary Producer Service, if there has been
 *                              no contact from the API.
 *  @param  properties          Optional properties for the new producer. Zero,
 *                              or a combination (bitwise OR) of:   \n
 *                                  RGMAProducerProps_LATEST and    \n
 *                                  RGMAProducerProps_HISTORY
 *  @param  storageType         The tuple storage type for the new producer.
 *                              One of:                             \n
 *                                  RGMAStorageType_MEMORY,         \n
 *                                  RGMAStorageType_DATABASE.
 *  @param  storageLocation     Details of permanent tuple store. May be NULL
 *                              if using temporary storage. Usually NULL.
 *  @param  resourcePP		On output, will contain pointer to new Primary
 *                              Producer.
 *
 *  @return 0 on success, -1 on error.
 */

extern int RGMA_createPrimaryProducer(int terminationInterval, int properties, int storageType, RGMAStorageLocation *storageLocation, RGMAResource **resourcePP);
#endif /* RGMA_V4_COMPAT */


#ifndef RGMA_V4_COMPAT /* v5 version */
/**
 *  Creates a new Secondary Producer resource.
 *
 *  Extracts the URL of the Secondary Producer Service from the R-GMA
 *  configuration file $RGMA_HOME/etc/rgma/rgma.conf. Both the environment
 *  variable, and the file, must exist.
 *
 *  @param  terminationInterval The interval (in seconds) after which this
 *                              Secondary Producer resource will be destroyed by
 *                              Secondary Producer Service, if there has been
 *                              no contact from the API.
 *  @param  properties          Optional properties for the new producer. Zero,
 *                              or a combination (bitwise OR) of:   \n
 *                                  RGMAProducerProps_LATEST and    \n
 *                                  RGMAProducerProps_HISTORY.
 *  @param  storageType         The tuple storage type for the new producer.
 *                              One of:                             \n
 *                                  RGMAStorageType_MEMORY,         \n
 *                                  RGMAStorageType_DATABASE.
 *  @param  storageLocation     Details of permanent tuple store. May be NULL
 *                              if using temporary storage. Usually NULL.
 *  @param  resourcePP		On output, will contain pointer to new Secondary
 *                              Producer.
 *
 *  @return 0 on success, -1 on error.
 */

extern int RGMA_createSecondaryProducer(int terminationInterval, int properties, int storageType, RGMAStorageLocation *storageLocation, RGMAResource **resourcePP);
#endif /* RGMA_V4_COMPAT */


#ifndef RGMA_V4_COMPAT /* v4 version */
/**
 *  Creates a new On-demand Producer resource.
 *
 *  Extracts the URL of the On-demand Producer Service from the R-GMA
 *  configuration file $RGMA_HOME/etc/rgma/rgma.conf. Both the environment
 *  variable, and the file, must exist.
 *
 *  @param  terminationInterval The interval (in seconds) after which this
 *                              On-demand Producer resource will be destroyed by
 *                              On-demand Producer Service, if there has been
 *                              no contact from the API.
 *  @param  uri                 The URI of the system that will respond to
 *                              queries (the form "socket://hostname:port").
 *  @param  resourcePP		On output, will contain pointer to new On-demand
 *                              Producer.
 *
 *  @return 0 on success, -1 on error.
 */

extern int RGMA_createOnDemandProducer(int terminationInterval, const char *uri, RGMAResource **resourcePP);
#endif /* RGMA_V4_COMPAT */


/**
 *  Re-connects the API to the remote resource at a given resource-endpoint,
 *  i.e. it creates and initialises a local resource structure to interface to
 *  the given resource-endpoint. Does not contact the server - you should call
 *  RGMA_showSignOfLife() after reconnect() if you want to keep the resource
 *  alive.
 *
 *  This function is useful to re-attach to a remote resource following
 *  termination of the program that originally it. The behviour of the system
 *  is undefined if you make multiple connections to the same resource.
 *
 *  @param  endpointP           Pointer to resource-endpoint of remote resource.
 *  @param  resourceType        R-GMA resource type:
 *                              one of
 *                                  RGMAResourceType_CONSUMER,
 *                                  RGMAResourceType_PRIMARYPRODUCER,
 *                                  RGMAResourceType_SECONDARYPRODUCER,
 *                                  RGMAResourceType_ONDEMANDPRODUCER.
 *  @param  resourcePP          On output, will contain pointer to new resource.
 *
 *  @return 0 on success, -1 on error.
 *  @see RGMA_getEndpoint().
 */

extern int RGMA_reconnect(RGMAResourceEndpoint *endpointP, int resourceType, RGMAResource **resourcePP);


/**
 *  Registers a Primary Producer in the Registry, as a producer for the
 *  specified table.
 *
 *  @param  resourceP              Pointer to the Primary Producer resource to
 *                                 register.
 *  @param  tableName		   The name of the table to declare.
 *  @param  predicate              An SQL WHERE clause defining the subset of
 *                                 the table that this producer will publish.
 *                                 May an empty string.
 *  @param  hrpSec                 Retention period (in seconds).
 *                                 Tuples older than this will be removed from
 *                                 the history tuple storage.
 *  @param  lrpSec                 Retention period (in seconds).
 *                                 Tuples older than this will never be returned
 *                                 in latest queries and will be removed from
 *                                 the latest tuple storage.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAPrimaryProducer_declareTable(RGMAResource *resourceP, const char *tableName, const char *predicate, int hrpSec, int lrpSec);


#ifndef RGMA_V4_COMPAT /* v5 version */
/**
 *  Publishes data for a single tuple (row) into a table, using an SQL INSERT
 *  statement.
 *
 *  @param  resourceP              Pointer to the Primary Producer resource
 *                                 through which to publish data.
 *  @param  insertStatement	   An SQL INSERT statement.
 *  @param  lrpSec                 Retention period (in seconds). Set this to
 *                                 0 to use the default set in
 *                                 RGMAPrimaryProducer_declareTable().
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAPrimaryProducer_insert(RGMAResource *resourceP, const char *insertStatement, int lrpSec);
#endif /* RGMA_V4_COMPAT */


#ifndef RGMA_V4_COMPAT /* v5 version */
/**
 *  Publishes data for multiple tuples (rows) into a table, using SQL INSERT
 *  statements.
 *
 *  @param  resourceP              Pointer to the Primary Producer resource
 *                                 through which to publish data.
 *  @param  numInserts             The number of tuples to insert.
 *  @param  insertStatements	   List of insert statements, one for each
 *                                 tuple.
 *  @param  lrpSec  Retention period (in seconds). Set this to
 *                                 0 to use the default set in
 *                                 RGMAPrimaryProducer_declareTable().
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAPrimaryProducer_insertList(RGMAResource *resourceP, int numInserts, char **insertStatements, int lrpSec);
#endif /* RGMA_V4_COMPAT */


/**
 *  Returns a list of names and database parameters of all permanent tuple
 *  stores belonging to the user (or all tuple stores in the service if it
 *  is running insecurely).
 *
 *  You should free the list when no longer required, by calling
 *  RGMA_freeTupleStoreList().
 *
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return RGMATupleStoreList containing zero or more tuple store definitions,
 *          or NULL on error.
 */

extern RGMATupleStoreList *RGMAPrimaryProducer_listTupleStores(RGMAException *exceptionP);


/**
 *  Drops one of user's permanent tuple stores. Tuple stores that are currently
 *  being used by a producer cannot be dropped.
 *
 *  @param  logicalName          Logical name of tuple store to be dropped.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAPrimaryProducer_dropTupleStore(const char *logicalName, RGMAException *exceptionP);


/**
 *  Queries Primary Producer's retention period for history tuples in
 *  specified table.
 *
 *  @param  resourceP              Pointer to the Primary Producer resource to
 *                                 query.
 *  @param  tableName              Table name to query.
 *
 *  @return Retention period (in seconds) or -1 on error.
 */

extern int RGMAPrimaryProducer_getHistoryRetentionPeriod(RGMAResource *resourceP, const char *tableName);


/**
 *  Queries Primary Producer's retention period for latest tuples in specified
 *  table.
 *
 *  @param  resourceP              Pointer to the Primary Producer resource to
 *                                 query.
 *  @param  tableName              Table name to query.
 *
 *  @return Retention period (in seconds) or -1 on error.
 */

extern int RGMAPrimaryProducer_getLatestRetentionPeriod(RGMAResource *resourceP, const char *tableName);


/**
 *  Registers a Secondary Producer in the Registry, as a producer for the
 *  specified table.
 *
 *  @param  resourceP              Pointer to the Secondary Producer resource
 *                                 to register.
 *  @param  tableName		   The name of the table to declare.
 *  @param  predicate              An SQL WHERE clause defining the subset of
 *                                 the table that this producer will publish.
 *                                 May an empty string.
 *  @param  hrpSec                 Retention period (in seconds).
 *                                 Tuples older than this will be removed from
 *                                 the history tuple storage.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASecondaryProducer_declareTable(RGMAResource *resourceP, const char *tableName, const char *predicate, int hrpSec);


/**
 *  Returns a list of names and database parameters of all permanent tuple
 *  stores belonging to the user (or all tuple stores in the service if it
 *  is running insecurely).
 *
 *  You should free the list when no longer required, by calling
 *  RGMA_freeTupleStoreList().
 *
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return RGMATupleStoreList containing zero or more tuple store definitions,
 *          or NULL on error.
 */

extern RGMATupleStoreList *RGMASecondaryProducer_listTupleStores(RGMAException *exceptionP);


/**
 *  Drops one of user's permanent tuple stores. Tuple stores that are currently
 *  being used by a producer cannot be dropped.
 *
 *  @param  logicalName          Logical name of tuple store to be dropped.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASecondaryProducer_dropTupleStore(const char *logicalName, RGMAException *exceptionP);


/**
 *  Queries Secondary Producer's retention period for history tuples in
 *  specified table.
 *
 *  @param  resourceP              Pointer to the Secondary Producer resource
 *                                 to query.
 *  @param  tableName              Table name to query.
 *
 *  @return Retention period (in seconds) or -1 on error.
 */

extern int RGMASecondaryProducer_getHistoryRetentionPeriod(RGMAResource *resourceP, const char *tableName);


/**
 *  Registers an On-demand Producer in the Registry, as a producer for the
 *  specified static table. A static table only supports static queries and
 *  has no time-stamps associated with its tuples.
 *
 *  @param  resourceP              Pointer to the On-demand Producer resource
 *                                 to register.
 *  @param  tableName		   The name of the table to declare.
 *  @param  predicate              An SQL WHERE clause defining the subset of
 *                                 the table that this producer will publish.
 *                                 May an empty string.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAOnDemandProducer_declareStaticTable(RGMAResource *resourceP, const char *tableName, const char *predicate);


/**
 *  Starts this consumer's query, using the mediator to find relevant
 *  producers to contact. The query will automatically abort after the
 *  specified time interval. You cannot prevent the query from timing out,
 *  but you can set a very large time interval. The query is run asynchronously.
 *  You should keep poping tuples from the consumer by calling
 *  RGMAConsumer_pop() repeatedly until it returns a result set with its
 *  endOfResults flag set to 1 (true). You can stop a query's execution by
 *  calling RGMAConsumer_abort(). Any un-popped tuples will be lost if a query
 *  is restarted.
 *
 *  @param  resourceP              Pointer to the Consumer resource whose
 *                                 query is to be started.
 *  @param  timeoutSec             Query timeout (in seconds).
 *
 *  @return 0 on success or -1 on error.
 *  @see RGMAConsumer_abort() and RGMAConsumer_startDirected().
 */

extern int RGMAConsumer_start(RGMAResource *resourceP, int timeoutSec);


/**
 *  Starts this consumer's query, contacting all producers in a list provided
 *  by the user. In all other respects, this function is identical to
 *  RGMAConsumer_start(). Since the mediator is not used, it is up to the user
 *  to ensure that the producers in the list can, taken together, give a
 *  complete and correct answer to the query.
 *
 *  @param  resourceP              Pointer to the Consumer resource whose
 *                                 query is to be started.
 *  @param  timeoutSec             Query timeout (in seconds).
 *  @param  numProducers           Number of producers to contact (at least 1).
 *  @param  producers              List of producer resource-endpoints to
 *                                 contact.
 *
 *  @return 0 on success or -1 on error.
 *  @see RGMAConsumer_start().
 */

extern int RGMAConsumer_startDirected(RGMAResource *resourceP, int timeoutSec, int numProducers, RGMAResourceEndpoint *producers);


/**
 *  Aborts a running query, after a call to start() or startDirected()). Does
 *  nothing if the query has already finished. Any tuples which had already
 *  been delivered to the consumer service may still be retrieved via pop.
 *
 *  @param  resourceP              Pointer to the Consumer resource which
 *                                 started the query.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMAConsumer_abort(RGMAResource *resourceP);


/**
 *  Determines whether a Consumer's query has aborted (either following a call
 *  to abort(), or because its query has timed out).
 *
 *  Note that as this involves a remote call, you should be careful to check
 *  for errors.
 *
 *  @param  resourceP              Pointer to the Consumer resource to query.
 *
 *  @return 1 if it has, 0 if it has not; -1 if an error occurred.
 */

extern int RGMAConsumer_hasAborted(RGMAResource *resourceP);


/**
 *  Retrieves at most maxCount tuples from a Consumer which have been returned
 *  by producers. The returned list will be empty if there are no tuples
 *  currently available. The final result set for a query will have its
 *  endOfResults flag set to 1 (true).
 *
 *  You should free the result set when no longer required, by calling
 *  RGMA_freeResultSet().
 *
 *  @param  resourceP              Pointer to the Consumer resource to query.
 *  @param  maxCount               The maximum number of tuples to retrieve.
 *
 *  @return Pointer to new result set, or NULL on error.
 *  @see RGMAResultSet, RGMA_printResultSet(), RGMA_freeResultSet().
 */

extern RGMAResultSet *RGMAConsumer_pop(RGMAResource *resourceP, int maxCount);


/**
 *  Queries the termination interval for a resource.
 *
 *  @param  resourceP           Pointer to the resource to query.
 *
 *  @return Termination interval (in seconds) or -1 on error.
 */

extern int RGMA_getTerminationInterval(RGMAResource *resourceP);


/**
 *  Indicates to a service that a resource should be kept alive. This
 *  method should be called periodically if there is no other contact with the
 *  resource through another API function.
 *
 *  @param  resourceP           Pointer to the resource to keep alive.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMA_showSignOfLife(RGMAResource *resourceP);


/**
 *  Schedules a (remote) resource for destruction. The resource will no longer
 *  be available through this API and will be destroyed by the service once
 *  it has finished interacting with other components. You should also free up
 *  any local memory associated with the resource using RGMA_freeResource().
 *
 *  @param  resourceP           Pointer to the resource to close.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMA_close(RGMAResource *resourceP);


/**
 *  Destroys a (remote) resource immediately. The resource will no longer be
 *  available to any component. You should also free up any local memory
 *  associated with the resource by calling RGMA_freeResource().
 *
 *  @param  resourceP           Pointer to the resource to destroy.
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMA_destroy(RGMAResource *resourceP);


/**
 *  Creates a new instance (replica) of a Registry for a specified virtual
 *  database in a Registry Service. This function would only normally be
 *  used by system administrators.
 *
 *  @param  vdbName           Virtual database name.
 *  @param  registryService   URL of registry service in which to create the
 *                            registry instance.
 *  @param  exceptionP        Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMARegistry_createRegistry(const char *vdbName, const char *registryService, RGMAException *exceptionP);


/**
 *  Destroys an instance (replica) of a Registry of a specified virtual
 *  database from a Registry Service. The Registry database is wiped. This
 *  function would only normally be used by system administrators.
 *
 *  @param  vdbName           Virtual database name.
 *  @param  registryService   URL of registry service from which to drop the
 *                            registry instance.
 *  @param  exceptionP        Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMARegistry_destroyRegistry(const char *vdbName, const char *registryService, RGMAException *exceptionP);


/**
 *  Returns details of all producers that are registered for a specified table
 *  in a specified virtual database.
 *
 *  @param  vdbName           Virtual database name.
 *  @param  tableName         Table name.
 *  @param  exceptionP        Pointer to exception (filled in on failure).
 *
 *  @return List of producers (may be empty) or NULL on error.
 */

extern RGMAProducerTableEntryList *RGMARegistry_getAllProducersForTable(const char *vdbName, const char *tableName, RGMAException *exceptionP);


/**
 *  Creates a new instance (replica) of a Schema for a specified virtual
 *  database in a Schema Service. This function would only normally be
 *  used by system administrators.
 *
 *  @param  vdbName           Virtual database name.
 *  @param  schemaService     URL of schema service in which to create the
 *                            schema instance.
 *  @param  exceptionP        Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_createSchema(const char *vdbName, const char *schemaService, RGMAException *exceptionP);


/**
 *  Destroys an instance (replica) of a Schema of a specified virtual
 *  database from a Schema Service. The Schema database is wiped. This
 *  function would only normally be used by system administrators.
 *
 *  @param  vdbName           Virtual database name.
 *  @param  schemaService     URL of schema service from which to drop the
 *                            schema instance.
 *  @param  exceptionP        Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_destroySchema(const char *vdbName, const char *schemaService, RGMAException *exceptionP);


/**
 *  Creates a new table in the schema of a specified virtual database. The
 *  table name, column names and column types must conform to the rules in the
 *  User Guide. A set of initial authorization rules for the table may also
 *  be specified. The format of the rules is described in the User Guide.
 *
 *  @param  vdbName               Virtual database name.
 *  @param  createTableStatement  SQL CREATE TABLE statement.
 *  @param  numRules              Number of authorization rules (may be 0).
 *  @param  rules                 Authorization rules (may be NULL).
 *  @param  exceptionP            Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_createTable(const char *vdbName, const char *createTableStatement, int numRules, char **rules, RGMAException *exceptionP);


/**
 *  Drops a table from the schema of a specified virtual database.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  tableName            Table to drop.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_dropTable(const char *vdbName, const char *tableName, RGMAException *exceptionP);


/**
 *  Creates a new index definition for a table in the schema of a specified
 *  virtual database. The index name and index definition must conform to the
 *  rules in the User Guide.
 *
 *  @param  vdbName               Virtual database name.
 *  @param  createIndexStatement  SQL CREATE INDEX statement.
 *  @param  exceptionP            Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_createIndex(const char *vdbName, const char *createIndexStatement, RGMAException *exceptionP);


/**
 *  Drops an index from the schema of a specified virtual database.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  indexName            Index to drop.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_dropIndex(const char *vdbName, const char *indexName, RGMAException *exceptionP);


/**
 *  Creates a new view in the schema of a specified virtual database. The
 *  view name and view definition must conform to the rules in the User Guide.
 *  A set of initial authorization rules for the view may also be specified.
 *  The format of the rules is described in the User Guide.
 *
 *  @param  vdbName               Virtual database name.
 *  @param  createViewStatement   SQL CREATE VIEW statement.
 *  @param  numRules              Number of authorization rules (may be 0).
 *  @param  rules                 Authorization rules (may be NULL).
 *  @param  exceptionP            Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_createView(const char *vdbName, const char *createViewStatement, int numRules, char **rules, RGMAException *exceptionP);


/**
 *  Drops a view from the schema of a specified virtual database.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  viewName             View to drop.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_dropView(const char *vdbName, const char *viewName, RGMAException *exceptionP);


/**
 *  Returns a list of all tables and views in the schema of a specified
 *  virtual database.
 *
 *  You should free the list returned by calling RGMA_freeStringList when
 *  it is no longer required.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return List of tables and views (may be empty) or NULL on error.
 */

extern RGMAStringList *RGMASchema_getAllTables(const char *vdbName, RGMAException *exceptionP);


/**
 *  Returns the column definitions of a table or view in the schema of a
 *  specified virtual database.
 *
 *  You should free the structure returned by calling RGMA_freeTableDefinition
 *  when it is no longer required.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  tableName            Table or view to query.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return Table definition or NULL on error.
 */

extern RGMATableDefinition *RGMASchema_getTableDefinition(const char *vdbName, const char *tableName, RGMAException *exceptionP);


/**
 *  Returns the list of indexes defined on a table in the schema of a
 *  specified virtual database.
 *
 *  You should free the list returned by calling RGMA_freeIndexList when
 *  it is no longer required.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  tableName            Table to query.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return List of indexes (may be empty) or NULL on error.
 */

extern RGMAIndexList *RGMASchema_getTableIndexes(const char *vdbName, const char *tableName, RGMAException *exceptionP);


/**
 *  Replaces the authorization rules on a table or view in the schema of a
 *  specified virtual database. The format of the rules is described in the
 *  User Guide.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  tableName            Table or view to update.
 *  @param  numRules             Number of authorization rules (may be 0 to
 *                               remove all existing rules).
 *  @param  rules                Authorization rules (may be NULL).
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success or -1 on error.
 */

extern int RGMASchema_setAuthorizationRules(const char *vdbName, const char *tableName, int numRules, char **rules, RGMAException *exceptionP);


/**
 *  Returns the authorization rules on a table or view in the schema of a
 *  specified virtual database. The format of the rules is described in the
 *  User Guide.
 *
 *  You should free the list returned by calling RGMA_freeStringList when
 *  it is no longer required.
 *
 *  @param  vdbName              Virtual database name.
 *  @param  tableName            Table or view to update.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return List of authorization rules (may be empty) or NULL on error.
 */

extern RGMAStringList *RGMASchema_getAuthorizationRules(const char *vdbName, const char *tableName,  RGMAException *exceptionP);


/**
 *  Sets the value of a service property - see the User Guide for a list of
 *  properties that can be set.
 *
 *  @param  service              Service (see getVersion()).
 *  @param  name                 Property name.
 *  @param  parameter            Property parameter (may be NULL).
 *  @param  value                New value for property.
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return 0 on success, or -1 on error.
 */

extern int RGMA_setProperty(int service, const char *name, const char *parameter, const char *value, RGMAException *exceptionP);


/**
 *  Returns the value of a service property - see the User Guide for a list of
 *  properties that can be queried and the values that they return.
 *
 *  You should free the string returned (by calling free()) when it is no
 *  longer required.
 *
 *  @param  service              Service (see getVersion()).
 *  @param  name                 Property name.
 *  @param  parameter            Property parameter (may be NULL).
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return Requested property string or NULL on error.
 */

extern char *RGMA_getProperty(int service, const char *name, const char *parameter, RGMAException *exceptionP);


/**
 *  Returns a string representing the version number of an R-GMA service.
 *
 *  You should free the string returned (by calling free()) when it is no
 *  longer required.
 *
 *  @param  service              Service - one of:                \n
 *                                  RGMAService_PRIMARYPRODUCER   \n
 *                                  RGMAService_SECONDARYPRODUCER \n
 *                                  RGMAService_ONDEMANDPRODUCER  \n
 *                                  RGMAService_CONSUMER          \n
 *                                  RGMAService_REGISTRY          \n
 *                                  RGMAService_SCHEMA            \n
 *  @param  exceptionP           Pointer to exception (filled in on failure).
 *
 *  @return Version number string or NULL on error.
 */

extern char *RGMA_getVersion(int service, RGMAException *exceptionP);


/* THESE REMAINING (PUBLIC) FUNCTIONS ARE HANDLED LOCALLY BY THE API) *********/


/**
 *  Gets the resource-endpoint (URL and id) for a resource, in an
 *  RGMAResourceEndpoint structure. You should free the memory associated with
 *  it (using RGMA_freeEndpoint()) when you are done with it.
 *
 *  @param  resourceP           Pointer to the resource to query.
 *
 *  @return Pointer to new resource-endpoint, or NULL on error.
 *  @see RGMAResourceEndpoint
 */

extern RGMAResourceEndpoint *RGMA_getEndpoint(RGMAResource *resourceP);


/**
 *  Returns a pointer to the most recent exception which occurred in a
 *  given resource. The pointer is only valid until the resource is freed.
 *
 *  @param  resourceP           Pointer to the resource to query.
 *
 *  @return Pointer to exception, or NULL if (and only if) no exception has
 *          occurred in the resource.
 *  @see    RGMAException
 */

extern const RGMAException *RGMA_getException(RGMAResource *resourceP);


/**
 *  Convenience function: returns a pointer to the requested value (as a string)
 *  from a result set. You must specify a row number and either a column name
 *  or column number (if you specify both, the column name will be used).
 *
 *  @todo Column matching should probably be case-insensitive.
 *
 *  @param  rs             Pointer to result set to search.
 *  @param  rowNum         Row number (in the range 0 to (rs->numRows - 1))
 *  @param  colNum         Column number (in the range 0 to (rs->numCols - 1))
 *  @param  colName        Column name (may be NULL).
 *
 *  @return Pointer to requested value, or NULL if not found.
 *  @see RGMAResultSet
 */

extern char *RGMA_getResultSetValue(RGMAResultSet *rs, int rowNum, int colNum, const char *colName);


/**
 *  Convenience function: prints the contents of a result set to a file stream.
 *  If you don't specify a stream (i.e. pass NULL), stderr will be used.
 *
 *  @param  stream         File stream to which to print result set.
 *  @param  rs             Pointer to result set to print.
 *
 *  @return Nothing.
 *  @see RGMAResultSet
 */

extern void RGMA_printResultSet(FILE *stream, RGMAResultSet *rs);


/**
 *  Frees all memory associated with an RGMAResource structure, but leaves the
 *  remote resource intact. If the API never re-connects to the remote
 *  resource, it will continue to exist and operate until it exceeds its
 *  termination interval. 
 *
 *  @param  resourceP           Pointer to the RGMAResource to free.
 *
 *  @return Nothing.
 *  @see RGMAResource.
 */

extern void RGMA_freeResource(RGMAResource *resourceP);


/**
 *  Frees all memory associated with an RGMAResourceEndpoint structure.
 *
 *  @param  endpointP           Pointer to the RGMAResourceEndpoint to free.
 *
 *  @return Nothing.
 *  @see RGMAResourceEndpoint
 */

extern void RGMA_freeEndpoint(RGMAResourceEndpoint *endpointP);


/**
 *  Frees all memory contained within an RGMAException structure (but does not
 *  free the exception structure itself). Note that if an API call succeeds,
 *  no exception will be written by the API and you should not call this
 *  function.
 *
 *  @param  exceptionP           Pointer to the RGMAException to free.
 *
 *  @return Nothing.
 *  @see RGMAException
 */

extern void RGMA_freeException(RGMAException *exceptionP);


/**
 *  Frees all memory associated with an RGMAStringList structure.
 *
 *  @param  stringListP       Pointer to the RGMAStringList to free.
 *
 *  @return Nothing.
 *  @see    RGMAStringList
 */

extern void RGMA_freeStringList(RGMAStringList *stringListP);


/**
 *  Frees all memory associated with an RGMAResultSet structure.
 *
 *  @param  resultSetP           Pointer to the RGMAResultSet to free.
 *
 *  @return Nothing.
 *  @see    RGMAResultSet
 */

extern void RGMA_freeResultSet(RGMAResultSet *resultSetP);


/**
 *  Frees all memory associated with an RGMATupleStoreList structure.
 *
 *  @param  tupleStoreListP       Pointer to the RGMATupleStoreList to free.
 *
 *  @return Nothing.
 *  @see    RGMATupleStoreList
 */

extern void RGMA_freeTupleStoreList(RGMATupleStoreList *tupleStoreListP);


/**
 *  Frees all memory associated with an RGMAProducerTableEntryList structure.
 *
 *  @param  producerTableEntryListP    Pointer to the RGMAProducerTableEntryList
 *                                     to free.
 *
 *  @return Nothing.
 *  @see    RGMAProducerTableEntryList
 */

extern void RGMA_freeProducerTableEntryList(RGMAProducerTableEntryList *producerTableEntryListP);


/**
 *  Frees all memory associated with an RGMAIndexList structure.
 *
 *  @param  indexListP       Pointer to the RGMAIndexList to free.
 *
 *  @return Nothing.
 *  @see    RGMAIndexList
 */

extern void RGMA_freeIndexList(RGMAIndexList *indexListP);


/**
 *  Frees all memory associated with an RGMATableDefinition structure.
 *
 *  @param  tableDefinitionP       Pointer to the RGMATableDefinition to free.
 *
 *  @return Nothing.
 *  @see    RGMATableDefinition
 */

extern void RGMA_freeTableDefinition(RGMATableDefinition *tableDefinitionP);


/* DEPRECATED FUNCTIONS *******************************************************/

#ifdef RGMA_V4_COMPAT

/** @deprecated Use the new version of this function. */
extern int RGMA_createPrimaryProducer(int terminationInterval, int properties, int storageType, RGMAStorageLocation *storageLocation, int numVOs, char **voNames, RGMAResource **resourcePP);

/** @deprecated Use the new version of this function. */
extern int RGMA_createSecondaryProducer(int terminationInterval, int properties, int storageType, RGMAStorageLocation *storageLocation, int numVOs, char **voNames, RGMAResource **resourcePP);

/** @deprecated Use the new version of this function. */
extern int RGMA_createOnDemandProducer(int terminationInterval, const char *uri, int numVOs, char **voNames, RGMAResource **resourcePP);

/** @deprecated Use the new version of this function. */
extern int RGMAPrimaryProducer_insert(RGMAResource *resourceP, const char *insertStatement);

/** @deprecated Use the new version of this function. */
extern int RGMAPrimaryProducer_insertList(RGMAResource *resourceP, int numInserts, char **insertStatements);

#endif /* RGMA_V4_COMPAT */

/* isExecuting and count are identical in v4 and v5 */

/** @deprecated Check endOfResults flag on RGMAResultSet instead. */
extern int RGMAConsumer_isExecuting(RGMAResource *resourceP);

/** @deprecated Check endOfResults flag on RGMAResultSet instead. */
extern int RGMAConsumer_count(RGMAResource *resourceP);

/******************************************************************************/


#ifdef __cplusplus
}
#endif

#endif /* RGMA_H */

/* End of file. */
